@@ -96,6 +96,9 @@ function convert(input, context) {
   if (!(input instanceof Buffer) && input !== FLUSH) {
     throw new Error('Bad argument.');  // Not a buffer or a string.
   }
+  if (context !== null && context.trailer !== null && input === FLUSH) {
+    throw errnoException('EINVAL', 'Incomplete character sequence.');
+  }
   if (context !== null && context.trailer !== null) {
     // Prepend input buffer with trailer from last chunk.
     var newbuf = new Buffer(context.trailer.length + input.length);
@@ -118,7 +121,8 @@ function convert(input, context) {
                                  output,
                                  output_start,
                                  output_size,
-                                 out);
+                                 out,
+                                 input === FLUSH);
     var input_consumed = out[0];
     var output_consumed = out[1];
     input_start += input_consumed;
